;
; SMS Dump - <inram.asm>
; Functions & data copied in RAM
;

; Main Loop -------------------------------------------------------------------
main_loop:

        ; Wait for V-Blank at first
        ; FIXME: This is incorrect, different behavior on NTSC and PAL/SECAM
        in      a, (VDP_VLINE)
        cp      $B9 ; 193
        jr      nz, main_loop

        ; Update blinking [WAITING] text
        ld      a, (VAR_blink_flag)
        inc     a
        ld      (VAR_blink_flag), a
        and     $08
        ld      hl, VRAM_BG_MAP + (23 + 22 * 32) * 2
        ld      b, 7
        ld      c, a
        call    bg_attr_set

        ; Check for button 1
        in      a, (PORT_INPUT1)

        bit     P1_UP_BIT, a
        jr      z, cursor_up
        bit     P1_DOWN_BIT, a
        jr      z, cursor_down
        bit     P1_LEFT_BIT, a
        jr      z, cursor_left
        bit     P1_RIGHT_BIT, a
        jr      z, cursor_right

        bit     P1_BUTTON1_BIT, a
        jp      z, copy_start

        bit     P1_BUTTON2_BIT, a
        jp      z, hexviewer_start

        jp      main_loop
;------------------------------------------------------------------------------
main_loop_wait_input_release:
        in      a, (PORT_INPUT1)
        cpl
        and     $3F
        jr      nz, main_loop_wait_input_release
        jp      main_loop
;------------------------------------------------------------------------------

cursor_down:
        ld      a, (VAR_cursor_pos)
        inc     a
        cp      9
        jr      nz, cursor_down_end
        xor     a
cursor_down_end:
        ld      (VAR_cursor_pos), a
        jr      cursor_end

cursor_up:
        ld      a, (VAR_cursor_pos)
        dec     a
        cp      $FF
        jr      nz, cursor_up_end
        ld      a, 8
cursor_up_end:
        ld      (VAR_cursor_pos), a
        jr      cursor_end

cursor_left:
        ld      hl, VAR_params_start
        ld      a, (VAR_cursor_pos)
        ld      d, 0
        ld      e, a
        add     hl, de
        ld      a, (hl)
        dec     a
        and     $03
        ld      (hl), a
        jr      cursor_end

cursor_right:
        ld      hl, VAR_params_start
        ld      a, (VAR_cursor_pos)
        ld      d, 0
        ld      e, a
        add     hl, de
        ld      a, (hl)
        inc     a
        and     $03
        ld      (hl), a
        jr      cursor_end

cursor_end:
        call    params_refresh
        jp      main_loop_wait_input_release

; DELAY_SMALL() ---------------------------------------------------------------
; Produce a small delay
;------------------------------------------------------------------------------
delay_small:
        push    af
        push    bc
        ld      bc, $8000
delay_small_loop:
        dec     bc
        ld      a, b
        or      c
        jr      nz, delay_small_loop
        pop     bc
        pop     af
        ret
;------------------------------------------------------------------------------

; COPY_START() ----------------------------------------------------------------
; Copying frontend - call COPY_PERFORM()
;------------------------------------------------------------------------------
copy_start:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        and     P1_BUTTON1
        jr      z, copy_start

        ; Print "COPYING" string
        ld      de, string_copying
        ld      hl, VRAM_BG_MAP + (23 + 22 * 32) * 2
        call    vdp_print

        ; Setup & perform the actual copy
        call    copy_setup
        call    copy_perform

        ; Print "OK" string
        ld      de, string_ok
        ld      hl, VRAM_BG_MAP + (23 + 22 * 32) * 2
        call    vdp_print

copy_end_wait_button:
        ; Wait for button press
        in      a, (PORT_INPUT1)
        and     P1_BUTTON1
        jr      nz, copy_end_wait_button
copy_end_wait_button_release:
        ; Wait for button release
        in      a, (PORT_INPUT1)
        and     P1_BUTTON1
        jr      z, copy_end_wait_button_release

        ; Print "WAITING" string again
        ld      de, string_waiting
        ld      hl, VRAM_BG_MAP + (23 + 22 * 32) * 2
        call    vdp_print

        ; Small delay
        call    delay_small

        jp      main_loop
;------------------------------------------------------------------------------

; COPY_SETUP() ----------------------------------------------------------------
; Setup copy variables
;------------------------------------------------------------------------------
copy_setup:
        ; Setup copy variables
        ld      d, 0

        ; SRC SLOT -----------------------
        ld      a, (VAR_param_0_src_slot)
        ld      e, a
        ld      hl, var_params_slot_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_src_slot), a

        ; DST SLOT -----------------------
        ld      a, (VAR_param_5_dst_slot)
        ld      e, a
        ld      hl, var_params_slot_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_dst_slot), a

        ; SRC MAPPER ---------------------
        ld      a, (VAR_param_1_src_mapper)
        rlca
        ld      e, a
        ld      hl, var_params_src_mapper_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_src_mapper), a
        inc     hl
        ld      a, (hl)
        ld      (VAR_copy_src_mapper+1), a

        ; DST MAPPER ---------------------
        ld      a, (VAR_param_6_dst_mapper)
        rlca
        ld      e, a
        ld      hl, var_params_dst_mapper_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_dst_mapper), a
        inc     hl
        ld      a, (hl)
        ld      (VAR_copy_dst_mapper+1), a

        ; SRC ADDRESS --------------------
        call    copy_setup_src_address

        ; DST ADDRESS --------------------
        call    copy_setup_dst_address

        ; SRC START ----------------------
        ld      a, (VAR_param_2_src_start)
        ld      e, a
        ld      hl, var_params_start_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_src_page_start), a

        ; DST START ----------------------
        ld      a, (VAR_param_7_dst_start)
        ld      e, a
        ld      hl, var_params_start_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_dst_page_start), a

        ; SIZE ---------------------------
        ld      a, (VAR_param_4_src_size)
        ld      e, a
        ld      hl, var_params_size_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_size_left), a

        ; Current Pages -----------------
        ld      a, (VAR_copy_src_page_start)
        ld      (VAR_copy_src_page_current), a
        ld      a, (VAR_copy_dst_page_start)
        ld      (VAR_copy_dst_page_current), a

        ret
;------------------------------------------------------------------------------
copy_setup_src_address:
        ld      a, (VAR_param_3_src_address)
        rlca
        ld      d, 0
        ld      e, a
        ld      hl, var_params_src_address_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_src_address), a
        inc     hl
        ld      a, (hl)
        ld      (VAR_copy_src_address+1), a
        ret
;------------------------------------------------------------------------------
copy_setup_dst_address:
        ld      a, (VAR_param_8_dst_address)
        rlca
        ld      d, 0
        ld      e, a
        ld      hl, var_params_dst_address_table
        add     hl, de
        ld      a, (hl)
        ld      (VAR_copy_dst_address), a
        inc     hl
        ld      a, (hl)
        ld      (VAR_copy_dst_address+1), a
        ret
;------------------------------------------------------------------------------

; COPY_PERFORM() --------------------------------------------------------------
; Perform the actual copy, based on parameters stored in RAM
;------------------------------------------------------------------------------
copy_perform:

; Copy each page (16 kb) ------------------------------------------------------
copy_perform_loop:

        xor     a
        ld      (VAR_copy_current_part), a

        ; Reset source reading address if a mapper was selected
        ld      a, (VAR_param_1_src_mapper)
        and     a
        jr      z, +3
        call    copy_setup_src_address

        ; Reset destination reading address if a mapper was selected
        ld      a, (VAR_param_6_dst_mapper)
        and     a
        jr      z, +3
        call    copy_setup_dst_address

; Copy each part (2 kb) -------------------------------------------------------
copy_perform_loop_part:

        ; 1. Select SRC slot
        ld      a, $EB
        out     (PORT_SLOT), a
        ld      a, (VAR_copy_src_slot)
        out     (PORT_SLOT), a

        ; 2. See if user selected a mapper
copy_perform_src_mapper:
        ld      a, (VAR_param_1_src_mapper)
        and     a
        jr      z, copy_perform_src_mapper_end        ; No mapper!
        ld      hl, (VAR_copy_src_mapper)

        ; 3. Map the current page on the mapper (if needed)
        ld      a, (VAR_copy_src_page_current)
        ld      (hl), a
copy_perform_src_mapper_end:

        ; 4. Setup reading address, destination address, length
        ld      hl, (VAR_copy_src_address)
        ld      de, RAM + $1000 ; Middle of RAM
        ld      bc, $800        ; 2 kb

        ; 5. Perform copy of 2 kb from source to RAM
        ldir

        ; 6. Save back current source address
        ld      (VAR_copy_src_address), hl

        ; 7. Select DST slot
        ld      a, $EB
        out     (PORT_SLOT), a
        ld      a, (VAR_copy_dst_slot)
        out     (PORT_SLOT), a

        ; 8. See if user selected a mapper
copy_perform_dst_mapper:
        ld      a, (VAR_param_6_dst_mapper)
        and     a
        jr      z, copy_perform_dst_mapper_end        ; No mapper!
        ld      hl, (VAR_copy_dst_mapper)

        ; 9. Map the current page on the mapper (if needed)
        ld      a, (VAR_copy_dst_page_current)
        ld      (hl), a
copy_perform_dst_mapper_end:

        ; 10. Setup source address, writing address, length
        ld      hl, RAM + $1000
        ld      de, (VAR_copy_dst_address)
        ld      bc, $800        ; 2 kb

        ; 11. Perform copy of 2 kb from RAM to destination
        ldir

        ; 11b. Now select BIOS slot, to avoid corruption on destination slot
        ld      a, $EB
        out     (PORT_SLOT), a

        ; 12. Save back current destination address
        ld      (VAR_copy_dst_address), de

        ; Increment "copy_current_part" and check if we reached part 8 (= 16 KB)
        ld      a, (VAR_copy_current_part)
        inc     a
        ld      (VAR_copy_current_part), a
        and     $8
        jr      z, copy_perform_loop_part

        ;----------------------------------------------------------------------

        ; Increment "*_copy_current", decrement "copy_size_left"
        ; and check if we reached our amount of page
        ld      a, (VAR_copy_src_page_current)
        inc     a
        ld      (VAR_copy_src_page_current), a
        ld      a, (VAR_copy_dst_page_current)
        inc     a
        ld      (VAR_copy_dst_page_current), a
        ld      a, (VAR_copy_size_left)
        dec     a
        ld      (VAR_copy_size_left), a
        jp      nz, copy_perform_loop

        ret
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; HEXVIEWER_COPY_SETUP()
; Setup copy parameters and adjust for hexviewer usage
;------------------------------------------------------------------------------
hexviewer_copy_setup:

        ; Setup copying parameters (since we're using them too)
        call    copy_setup

        ; Limit size to $13 (to fit on screen)
        ld      a, (VAR_copy_size_left)
        cp      $14
        jr      c, +2
        ld      a, $14
        ld      (VAR_copy_size_left), a
        
        ; Overwrite src_page start/current
        ld      a, (VAR_hexviewer_page_start)
        ld      (VAR_copy_src_page_start), a
        ld      (VAR_copy_src_page_current), a
        
        ret
        
; HEXVIEWER_START() -----------------------------------------------------------
; Embedded hexadecimal data viewer
;------------------------------------------------------------------------------
hexviewer_start:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        and     P1_BUTTON2
        jr      z, hexviewer_start

        ; Start at page 0, byte 0
        xor     a
        ld      (VAR_hexviewer_page_start), a
        ld      hl, 0
        ld      (VAR_hexviewer_byte_start), hl

        ; Clear background map
        call    bg_clear

        ; Print Hexviewer layout
        ld      de, string_hexviewer_top
        ld      hl, VRAM_BG_MAP
        call    vdp_print
        ld      de, string_hexviewer_bottom
        ld      hl, VRAM_BG_MAP + (22 * 32) * 2
        call    vdp_print

hexviewer_display:

        ; Setup display parameters
        call    hexviewer_copy_setup

; Display 8-bytes from each page ----------------------------------------------

hexviewer_display_loop:

        ; Reset source reading address if a mapper was selected
        ld      a, (VAR_param_1_src_mapper)
        and     a
        jr      z, +3
        call    copy_setup_src_address

        ; 1. Select SRC slot
        ld      a, $EB
        out     (PORT_SLOT), a
        ld      a, (VAR_copy_src_slot)
        out     (PORT_SLOT), a

        ; 2. See if user selected a mapper
        ld      a, (VAR_param_1_src_mapper)
        and     a
        jr      z, hexviewer_perform_src_mapper_end        ; No mapper!
        ld      hl, (VAR_copy_src_mapper)

        ; 3. Map the current page on the mapper (if needed)
        ld      a, (VAR_copy_src_page_current)
        ld      (hl), a
hexviewer_perform_src_mapper_end:

        ; 4. Compute 4-byte addition checksum for all 16 kb of the page
        ; Checksum is computed in IX
        ld      hl, (VAR_copy_src_address)
        ld      bc, $0040   ; 16 kbytes ($4000)
        ld      ix, $0000   ; Checksum starts at 0
        ld      d, $00

hexviewer_checksum_loop:
        ld      e, (hl)                 ; 7
        add     ix, de                  ; 15
        inc     hl                      ; 6
        djnz    hexviewer_checksum_loop ; 13/8
        dec     c
        jr      nz, hexviewer_checksum_loop
        
        ld      (VAR_hexviewer_checksum), ix

        ; 5. Setup reading address, destination address, length
        ld      hl, (VAR_copy_src_address)
        ld      bc, (VAR_hexviewer_byte_start)
        add     hl, bc
        ld      de, RAM + $1000 ; Middle of RAM
        ld      bc, $8          ; 8 bytes

        ; 6. Perform copy of 8 bytes from source to RAM
        ldir

        ; 6b. Increment address to reach next page
        ld      de, $3FF8
        add     hl, de

        ; 7. Save back current source address
        ld      (VAR_copy_src_address), hl

        ; 8. Now select BIOS slot, to avoid corruption on destination slot
        ld      a, $EB
        out     (PORT_SLOT), a

        ; 9. Print out format string with page number + 8 first bytes
        call    hexviewer_string_print

        ;----------------------------------------------------------------------

        ; Increment "*_copy_current", decrement "copy_size_left"
        ; and check if we reached our amount of page
        ld      a, (VAR_copy_src_page_current)
        inc     a
        ld      (VAR_copy_src_page_current), a
        ld      a, (VAR_copy_size_left)
        dec     a
        ld      (VAR_copy_size_left), a
        jp      nz, hexviewer_display_loop

hexviewer_input_loop:

        ; Check inputs
        in      a, (PORT_INPUT1)
        ld      b, a

        ; Button 1 = Refresh
        bit     P1_BUTTON1_BIT, b
        jr      z, hexviewer_display
        
        ; Button 2 = Quit
        bit     P1_BUTTON2_BIT, b
        jr      z, hexviewer_end
        
        ; Up/Down = Scroll
        bit     P1_UP_BIT, b
        jr      z, hexviewer_input_scroll_up
        bit     P1_DOWN_BIT, b
        jr      z, hexviewer_input_scroll_down
        bit     P1_LEFT_BIT, b
        jr      z, hexviewer_input_scroll_left
        bit     P1_RIGHT_BIT, b
        jr      z, hexviewer_input_scroll_right

        jr      hexviewer_input_loop

; Scroll up
hexviewer_input_scroll_up:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        bit     P1_UP_BIT, a
        jr      z, hexviewer_input_scroll_up
        
        ; Load current 'start' position
        ; Resetup copy and save 'start-1'
        ld      a, (VAR_hexviewer_page_start)
        cp      0
        jr      z, hexviewer_input_loop
        dec     a
        ld      (VAR_hexviewer_page_start), a
        jp      hexviewer_display
        
; Scroll down
hexviewer_input_scroll_down:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        bit     P1_DOWN_BIT, a
        jr      z, hexviewer_input_scroll_down

        ; Load current 'start' position
        ; Resetup copy and save 'start+1'
        ld      a, (VAR_hexviewer_page_start)
        inc     a
        ld      (VAR_hexviewer_page_start), a
        jp      hexviewer_display
 
; Scroll left
hexviewer_input_scroll_left:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        bit     P1_LEFT_BIT, a
        jr      z, hexviewer_input_scroll_left

        ; Offset -= 1
        ld      hl, (VAR_hexviewer_byte_start)
        dec     hl
        ld      (VAR_hexviewer_byte_start), hl
        jp      hexviewer_display

; Scroll right
hexviewer_input_scroll_right:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        bit     P1_RIGHT_BIT, a
        jr      z, hexviewer_input_scroll_right

        ; Offset -= 1
        ld      hl, (VAR_hexviewer_byte_start)
        inc     hl
        ld      (VAR_hexviewer_byte_start), hl
        jp      hexviewer_display

hexviewer_end:

        ; Wait for button release
        in      a, (PORT_INPUT1)
        bit     P1_BUTTON2_BIT, a
        jr      z, hexviewer_end

        ; Small delay
        call    delay_small

        ; Print Layout again
        call    bg_clear
        ld      de, string_layout
        ld      hl, VRAM_BG_MAP + (0 + 0 * 32) * 2
        call    vdp_print

        ; Refresh parameters
        call    params_refresh

        ; Return to main loop
        jp      main_loop
;------------------------------------------------------------------------------


; HEXVIEWER_PRINT_STRING() ----------------------------------------------------
; Print single string line corresponding to current status
hexviewer_string_print:

        ld      hl, VAR_hexviewer_string

        ld      a, (VAR_copy_src_page_current)
        ld      b, a
        call    hexviewer_string_print_hexnumber

        ld      (hl), ':'
        inc     hl
        ld      (hl), ' '
        inc     hl

        ld      c, 7
        ld      de, RAM + $1000
hexviewer_string_print_loop:

        ; Append one hexadecimal byte
        ld      a, (de)
        inc     de
        ld      b, a
        call    hexviewer_string_print_hexnumber

        ; Append space        
        ld      (hl), ' '
        inc     hl
        
        dec     c
        jr      nz, hexviewer_string_print_loop

        ; Append checksum
        ld      (hl), '='
        inc     hl
        ld      ix, VAR_hexviewer_checksum
        ld      b, (ix+1)
        call    hexviewer_string_print_hexnumber
        ld      b, (ix+0)
        call    hexviewer_string_print_hexnumber

        ; Print string to screen at corresponding location
        ; Y = (current-start) + 2
        ld      (hl), 0
        ld      a, (VAR_copy_src_page_start)
        ld      b, a
        ld      a, (VAR_copy_src_page_current)
        sub     b
        add     a, 2
        ld      b, a
        ; X = 1
        ld      a, 1
        ld      c, a
        call    bg_addr_get
        ld      de, VAR_hexviewer_string
        call    vdp_print

        ; Print offset
        ld      hl, VAR_hexviewer_string
        ld      a, (VAR_hexviewer_byte_start+1)
        ld      b, a
        call    hexviewer_string_print_hexnumber
        ld      a, (VAR_hexviewer_byte_start+0)
        ld      b, a
        call    hexviewer_string_print_hexnumber
        ld      (hl), 0
        
        ld      b, 23
        ld      c, 9
        call    bg_addr_get
        ld      de, VAR_hexviewer_string
        call    vdp_print


        ret
;------------------------------------------------------------------------------
hexviewer_string_print_hexnumber:

        ld      a, b
        
        ; First digit
        rra
        rra
        rra
        rra
        and     $F
        add     a, 48   ; Ascii for '0'
        cp      48+10
        jr      c, +2
        add     a, 7
        ld      (hl), a
        inc     hl

        ; Second digit
        ld      a, b
        and     $F
        add     a, 48   ; Ascii for '0'
        cp      48+10
        jr      c, +2
        add     a, 7
        ld      (hl), a
        inc     hl

        ret
;------------------------------------------------------------------------------
;hexviewer_string_template:
;        .DB     "XX : 00 11 22 33 44 55 66 77  ", 0
;hexviewer_string_template_end:
;------------------------------------------------------------------------------


; PARAMS_REFRESH() ----------------------------------------------------------
; Refresh background attributes to highlight selected parameters
;----------------------------------------------------------------------------
params_refresh:
        ; Clear all attributes in BG (six by six)
        ld      b, 12/6
        ld      hl, VRAM_BG_MAP + (7 * 32) * 2
        ld      de, 64*6
params_refresh_clear_loop:
        push    bc
        push    hl
        ld      b, 32*6
        ld      c, $00
        call    bg_attr_set
        pop     hl
        pop     bc
        add     hl, de
        djnz    params_refresh_clear_loop

        ; Now refresh all attributes in BG
        ld      bc, 0
        ld      hl, VAR_params_start
        ld      de, var_params_y_coord

params_refresh_loop:
        push    bc
        push    de
        push    hl

params_refresh_cursor:
        ld      a, (VAR_cursor_pos)
        cp      c
        jr      nz, params_refresh_cursor_end
        ld      a, (de)
        ld      b, a
        xor     a
        ld      c, a
        call    bg_addr_get
        ld      b, 8
        ld      c, $08
        call    bg_attr_set
        pop     hl
        pop     de
        pop     bc
        dec     sp
        dec     sp
        dec     sp
        dec     sp
        dec     sp
        dec     sp
params_refresh_cursor_end:

        ; X-Coordinate
        ld      a, (hl)         ; Load parameter in A
        ld      b, a            ; Multiply A by 6
        add     a, b            ;
        add     a, b            ;
        add     a, a            ;
        add     a, 9            ; Add 9
        ld      c, a

        ; Y-Coordinate
        ld      a, (de)
        ld      b, a

        ; Get screen coordinate
        call    bg_addr_get

        ; Now call bg_attr_set
        ld      b, 4
        ld      c, $08
        call    bg_attr_set

        pop     hl
        pop     de
        pop     bc

        inc     de
        inc     hl

        inc     c
        ld      a, c
        cp      9
        jr      nz, params_refresh_loop

        ret
;------------------------------------------------------------------------------

; BG_CLEAR() ----------------------------------------------------------------
; Clear background
;----------------------------------------------------------------------------
bg_clear:
        ld      hl, VRAM_BG_MAP|$4000
        ld      a, l
        out     (VDP_ADDR), a
        ld      a, h
        out     (VDP_ADDR), a
        ld      bc, 32*24*2
bg_clear_loop:
        xor     a
        out     (VDP_DATA), a
        dec     bc
        ld      a, b
        or      c
        jr      nz, bg_clear_loop
        ret
;----------------------------------------------------------------------------


; BG_ADDR_GET() -------------------------------------------------------------
; Get background address for corresponding position
;----------------------------------------------------------------------------
; b = y
; c = x
;----------------------------------------------------------------------------
; Return:
; hl = address in character map
;----------------------------------------------------------------------------
bg_addr_get:
        push    de

        ld      hl, VRAM_BG_MAP         ; Base Address -----

        ld      a, c                    ; X ----------------
        and     a                       ; clear carry
        rla                             ;
        ld      d, 0                    ;
        ld      e, a                    ;
        add     hl, de                  ;

        ld      de, 64                  ; Y ----------------
bg_addr_get_loop:                       ;
        add     hl, de                  ;
        djnz    bg_addr_get_loop        ;

        pop     de
        ret
;----------------------------------------------------------------------------


; BG_ATTR_SET() -------------------------------------------------------------
; Change background attributes of a given zone
;----------------------------------------------------------------------------
; hl = address in character map
; b  = length
; c  = new attribute
;----------------------------------------------------------------------------
bg_attr_set:
        inc     l
        ld      a, h
        or      $40
        ld      h, a

bg_attr_set_loop:
        ld      a, l                    ; Note: avoid using RST $28
        out     (VDP_ADDR), a           ; as we want this code to be
        ld      a, h                    ; executed without the cart.
        out     (VDP_ADDR), a
        ld      a, c
        out     (VDP_DATA), a
        inc     hl
        inc     hl
        djnz    bg_attr_set_loop

        ret
;----------------------------------------------------------------------------

; VDP_PRINT() ---------------------------------------------------------------
; Print a string to character map
; Currently, font is stored from tile 80
;----------------------------------------------------------------------------
; hl = address in character map
; de = zero terminated string address
;----------------------------------------------------------------------------
; Trash AF, BC, DE, HL
;----------------------------------------------------------------------------
vdp_print:
                ld      a, l                    ; Note: avoid using RST $28
                out     (VDP_ADDR), a           ; as we want this code to be
                ld      a, h                    ; executed without the cart.
                or      $40
                out     (VDP_ADDR), a
vdp_print_loop:
                ld      a, (de)
                and     a                       ; end if we found a zero
                jr      z, vdp_print_loop_end
                inc     de
                ld      b, a
                cp      13                      ; test for \n
                jr      z, vdp_print_new_line
                add     a, -32
                out     (VDP_DATA), a
                xor     a
                push    ix
                pop     ix
                out     (VDP_DATA), a
                jr      vdp_print_loop
vdp_print_new_line:
                ld      bc, 64
                add     hl, bc

                ld      a, l                    ; Note: avoid using RST $28
                out     (VDP_ADDR), a           ; as we want this code to be
                ld      a, h                    ; executed without the cart.
                or      $40
                out     (VDP_ADDR), a

                jr      vdp_print_loop
vdp_print_loop_end:
                ret
;----------------------------------------------------------------------------

; STRINGS ---------------------------------------------------------------------
string_waiting:
        .DB     "WAITING", 0
string_copying:
        .DB     "COPYING", 0
string_ok:
        .DB     "  OK!  ", 0
string_hexviewer_top:
        .DB     "   SOURCE PAGES HEADER VIEWER   ",     13
        .DB     "--------------------------------",     0
string_hexviewer_bottom:
        .DB     "--------------------------------",     13
        .DB     " OFFSET: XXXX", 0
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------

; CONFIGURABLE DATA -----------------------------------------------------------
; Includes:
;  - string_layout
;  - var_params_default
;  - var_params_x_coord
;  - var_params_y_coord
;  - var_params_slot_table
;  - var_params_mapper_table
;  - var_params_address_table
;  - var_params_start_table
;  - var_params_size_table
.INCLUDE "config.asm"
;------------------------------------------------------------------------------

